local L_System = {}
-- Lindenmayer system for Displaying Reiteratively Generated Fractals
--	a, b, c & d cause movement  (changed after each level)
-- 	m repeats previous movement (uneffected by level change)
--	n movement is not drawn     (uneffected by level change)
--	r & l change direction of movement for following character
--	start (axiom) contains starting movement list
--	ruleA, ruleB... contains replacement sequence for current a, b... (next level)
-- Script
--	angle changes drawing direction (effects l & r)
--  size controls fractal screen size
--  levels determines number of irrerations to perform

local gr, pi, sin, cos = love.graphics,math.pi, math.sin, math.cos

function L_System.loadfractal( fract ) 	-- creates table containing path for each level
	local path      = {}
	path[ 0 ] = fract.start				-- initial string
	numlevels = fract.levels
	leveldir  = 1						-- (1)increasing or (-1)decreasing level
	level 	  =	1						-- start at level 1
	for level = 1, numlevels do 		-- load table with all levels
		path[level] = L_System.expandCode( path, level )
		end
	return path
	end

function L_System.expandCode( frac, level )-- adds next level to previous one
	local w = ""
	for i = 1, #frac[ level-1 ] do
		local chr = string.sub( frac[ level-1 ], i, i )
		if chr == "a" then w = w..fractal[ selection ].ruleA
		elseif chr == "b" then w = w..fractal[ selection ].ruleB
		elseif chr == "c" then w = w..fractal[ selection ].ruleC
		elseif chr == "d" then w = w..fractal[ selection ].ruleD
		elseif chr == "m" or chr == "n" or chr == "r" or chr == "l"
					  or chr == "+" or chr == "-" or chr == "["
					  or chr == "]"
			then  						-- filter out extraneous characters
				w = w..chr
			end
		end
	return w
	end

function L_System.calctrig( angle )
	return sin( angle + pi/2 ), cos( angle + pi/2 )
	end

function L_System.range( x, y )  		-- store object limits
	if x > L_System.xmax then L_System.xmax = x end
	if x < L_System.xmin then L_System.xmin = x end
	if y > L_System.ymax then L_System.ymax = y end
	if y < L_System.ymin then L_System.ymin = y end
	end

function L_System.scaleFactors() 		-- scaling factors for screen display
	local scale = {fractal[selection].size / ( L_System.xmax - L_System.xmin ),
			fractal[selection].size / ( L_System.ymax - L_System.ymin )}
	local ave   = {( L_System.xmax + L_System.xmin ) / 2, ( L_System.ymax + L_System.ymin ) / 2}
	return scale, ave
	end

function L_System.calculateCoordinates( code )  -- translate l-code into x,y steps
	local coord = {}
	local x, y, angle = L_System.reset()
	local anglestep = fractal[ selection ].angle * pi / 180
	for i = 1, #code do
		local chr = string.sub( code, i, i )
		if chr == "n" then
			local s, c = L_System.calctrig()
			local dx = x * s - y * c
			local dy = x * s + y * c
			x, y = x + dx, y + dy
			end
		if chr == "r" then  angle = angle + anglestep end
		if chr == "l" then  angle = angle - anglestep end
		if chr == "[" then savex = x savey = y end 		-- allows branching
		if chr == "]" then x = savex y = savey end
		if chr == "a" or chr =="b" or chr == "m" then	-- draw movement but changed on looping
			local dx, dy = L_System.calctrig( angle )
			coord[ #coord+1 ] = { x, y }
			L_System.range( x, y )						-- keep track of data limits (for screen display)
			x, y = x + dx, y + dy
			end
		end
	coord[ #coord+1 ] = { x, y }
	L_System.range( x, y )
	return coord
	end

function L_System.showFractal( data )
	local scale, ave = L_System.scaleFactors()
	for i = 1, #data-1 do
		local px1 = scale[1] * ( data[i][1]   - ave[1] ) + centerx
		local py1 = scale[2] * ( data[i][2]   - ave[2] ) + centery
		local px2 = scale[1] * ( data[i+1][1] - ave[1] ) + centerx
		local py2 = scale[2] * ( data[i+1][2] - ave[2] ) + centery
		gr.line( px1, py1, px2, py2 )
		end
	end

function L_System.reset()
	L_System.xmin,L_System.xmax,L_System.ymin,L_System.ymax = 0,0,0,0
	return 0,0,math.pi  end		-- x,y,angle set to zero

return L_System
